clear all
set more off
set scheme s2manual

use spdaily
sjlog using logs/returns2, replace
generate return2 = return^2
tsset date
tsline return2
sjlog close, replace
graph export eps/Returns2.eps, replace
graph export svg/Returns2.svg, replace

sjlog using logs/ARCH1, replace
tsset newdate
arch return, arch(1) nolog
estimates store ARCH
sjlog close, replace

sjlog using logs/ARCHRobust, replace
arch return, arch(1) vce(robust) nolog
estimates store ARCHRobust
estimates table ARCH ARCHRobust, se
sjlog close, replace

* ARCH(1) Student-t
sjlog using logs/ARCHT, replace
arch return, arch(1) distribution(t) nolog
estimates store ARCH1T
sjlog close, replace

sjlog using logs/ARCHTIC, replace
estimates stats ARCH ARCH1T
sjlog close, replace

* ARCH(1) GED
sjlog using logs/ARCHGED, replace
arch return, arch(1) distribution(ged) nolog
estimates store ARCH1GED
sjlog close, replace

sjlog using logs/ARCHGEDIC, replace
estimates stats ARCH ARCH1T ARCH1GED
sjlog close, replace

* AR(2)-ARCH(1) Student-t
sjlog using logs/ARCHTARMA, replace
arch return, arch(1) distribution(t) ar(1/2) nolog
sjlog close, replace

sjlog using logs/ARCHTARMAIC, replace
estat ic
sjlog close, replace

* Weekend effect
sjlog using logs/hetWeekEffect, replace
generate dayofweek = dow(newdate)
generate monday = 0
replace monday = 1 if dayofweek == 1
generate friday = 0
replace friday = 1 if dayofweek == 5
arch return, arch(1) distribution(t) ar(1/2) het(monday friday) nolog
sjlog close, replace

sjlog using logs/hetWeekEffectIC, replace
estat ic
sjlog close, replace

* postestimation
sjlog using logs/ARCH1StdResiduals, replace
arch return, arch(1) nolog
predict residual, residuals
predict variance, variance
generate stdresidual = residual/variance^0.5
sjlog close, replace

sjlog using logs/ARCH1Residuals, replace
regress stdresidual
estat archlm, lags(1,5,10)
sjlog close, replace

ac stdresidual
graph export eps/ARCH1ResidualsAC.eps, replace
graph export svg/ARCH1ResidualsAC.svg, replace

sjlog using logs/HistogramResiduals, replace
histogram stdresidual, normal normopts(lcolor(gs13)) kdensity
sjlog close, replace
graph export eps/ARCH1ResidualsHist.eps, replace
graph export svg/ARCH1ResidualsHist.svg, replace

* ARCH(p)
sjlog using logs/ARCH10, replace
drop residual variance stdresidual
arch return, arch(1/10) nolog
sjlog close, replace

sjlog using logs/ARCH10IC, replace
estat ic
sjlog close, replace

sjlog using logs/ARCH10, replace
arch return, arch(1/10) nolog
sjlog close, replace

sjlog using logs/ARCH10Residuals, replace
predict residual, residuals
predict variance, variance
generate stdresidual = residual/variance^0.5
regress stdresidual
estat archlm, lags(1,5,10)
drop residual variance stdresidual
sjlog close, replace

* GARCH(1,1)
sjlog using logs/GARCH11, replace
arch return, arch(1) garch(1) nolog
sjlog close, replace

sjlog using logs/GARCH11_IC, replace
estat ic
sjlog close, replace

sjlog using logs/GARCH11Residuals, replace
predict residual, residuals
predict variance, variance
generate stdresidual = residual/variance^0.5
regress stdresidual
estat archlm, lags(1,5,10)
drop residual variance stdresidual
sjlog close, replace


* GARCH(2,2)
sjlog using logs/GARCH22, replace
arch return, arch(1/2) garch(1/2) nolog
predict residual, residuals
predict variance, variance
generate stdresidual = residual/variance^0.5
regress stdresidual
estat archlm, lags(1,5,10)
sjlog close, replace

sjlog using logs/GARCH22IC, replace
quietly arch return, arch(1/2) garch(1/2) nolog
estat ic
sjlog close, replace

ac stdresidual
graph export eps/GARCH22ResidualsAC.eps, replace
graph export svg/GARCH22ResidualsAC.svg, replace

sjlog using logs/GARCHHistogramResiduals, replace
histogram stdresidual, normal normopts(lcolor(gs13)) kdensity
sjlog close, replace
graph export eps/GARCH22ResidualsHist.eps, replace
graph export svg/GARCH22ResidualsHist.svg, replace

* AR(2)-GARCH(2,2) distribution t
sjlog using logs/ARMAGARCH22T, replace
drop residual variance stdresidual
arch return, arch(1/2) garch(1/2) ar(1/2) distribution(t) nolog
sjlog close, replace

sjlog using logs/ARMAGARCH22TIC, replace
estat ic
sjlog close, replace

* GARCH(1,1) on monthly data
use spmonthly, clear
sjlog using logs/GARCH11Monthly, replace
tsset time
arch return, arch(1) garch(1) nolog
sjlog close, replace

* GARCH in MEAN
use spdaily
sjlog using logs/GARCHM, replace
tsset newdate
arch return, arch(1) garch(1) archm archmlags(1) nolog
sjlog close, replace

* FORECAST on AR(2)-GARCH(2,2) distribution t
sjlog using logs/MeanForecast, replace
arch return, arch(1/2) garch(1/2) ar(1/2) distribution(t) nolog
predict mean, xb
tsset date
tsline return mean
sjlog close, replace
*lcolor() option used for shading in book
tsline return mean, lcolor(black gs12)
graph export eps/MeanForecast.eps, replace
graph export svg/MeanForecast.svg, replace

sjlog using logs/VarianceForecast, replace
predict variance, variance
tsline variance
sjlog close, replace
graph export eps/VarianceForecast.eps, replace
graph export svg/VarianceForecast.svg, replace

* Sign bias test
sjlog using logs/SBTTestPreliminary, replace
tsset newdate
arima return, ar(1/2) nolog
predict residuals, residual
generate residuals2 = residuals^2
sjlog close, replace

sjlog using logs/SBTTest, replace
generate SBT = (residuals<0)
generate NSBT = residuals*(residuals < 0)
generate PSBT = residuals*(residuals > 0)
regress residuals2 SBT NSBT PSBT
sjlog close, replace

* SAARCH(1,1)
sjlog using logs/SAARCH, replace
arch return, arch(1) garch(1) saarch(1) nolog
estimates store SAARCH
sjlog close, replace

* TGARCH(1,1)
sjlog using logs/TGARCH, replace
arch return, abarch(1) atarch(1) sdgarch(1) nolog
estimates store TGARCH
sjlog close, replace

* EGARCH(1,1)
*sjlog using logs/EGARCH11, replace
*arch return, egarch(1) earch(1)
*sjlog close, replace

*sjlog using logs/EGARCH11BlackMonday, replace
*arch return if blackmonday==0, egarch(1) earch(1) nolog
*estimates store EGARCH
*sjlog close, replace

* GJR(1,1)
sjlog using logs/GJR, replace
arch return, arch(1) garch(1) tarch(1) nolog
estimates store GJR
sjlog close, replace

* APARCH(1,1)
sjlog using logs/APARCH11, replace
arch return, aparch(1) pgarch(1) nolog
estimates store APARCH
sjlog close, replace

* summary
*estimates table SAARCH TGARCH EGARCH GJR APARCH, stats(aic bic)
sjlog using logs/GARCHComparison, replace
estimates table SAARCH TGARCH GJR APARCH, stats(aic bic)
sjlog close, replace

************* NEWS IMPACT CURVE ****************************************
sjlog using logs/gen_et, replace
generate et = (_n-8000)/2000
sjlog close, replace

sjlog using logs/NIC, replace
* GARCH(1,1)
arch return, arch(1) garch(1) nolog
predict varianceGARCH, variance at (et 1)
label variable varianceGARCH "VarianceGARCH"
* TGARCH(1,1)
arch return, abarch(1) atarch(1) sdgarch(1) nolog
predict varianceTGARCH, variance at (et 1)
label variable varianceTGARCH "VarianceTGARCH"
* GJR(1,1)
arch return, arch(1) garch(1) tarch(1) nolog
predict varianceGJR, variance at (et 1)
label variable varianceGJR "VarianceGJR"
line varianceGARCH varianceTGARCH varianceGJR et in 2/l, m(i) c(l) title(News response function)
sjlog close, replace
graph export eps/NIC.eps, replace
graph export svg/NIC.svg, replace

************* FORECAST ********************************************
sjlog using logs/Forecast, replace
* GARCH
arch return if date >= mdy(01,01,2013) & date < = mdy(11,30,2013), arch(1) garch(1)
predict var_GARCH_Forecast if date  >= mdy(01,01,2013), variance dynamic(mdy(11,30,2013))
* TGARCH
arch return if date >= mdy(01,01,2013) & date < = mdy(11,30,2013), abarch(1) atarch(1) sdgarch(1) nolog
predict var_TGARCH_Forecast if date  >= mdy(01,01,2013), variance dynamic(mdy(11,30,2013))
* GJR
arch return if date >= mdy(01,01,2013) & date < = mdy(11,30,2013), arch(1) garch(1) tarch(1) nolog
predict var_GJR_Forecast if date  >= mdy(01,01,2013), variance dynamic(mdy(11,30,2013))
generate return2 = return^2
tsset date
tsline return2 var_GARCH_Forecast var_TGARCH_Forecast var_GJR_Forecast if date>=mdy(01,01,2013), tline(30nov2013)
sjlog close, replace
tsline return2 var_GARCH_Forecast var_TGARCH_Forecast var_GJR_Forecast if date>=mdy(01,01,2013), tline(30nov2013) legend(col(1)) ylabel(, labsize(small)) ylabel(, labsize(small))
graph export eps/ForecastComparison.eps, replace
graph export svg/ForecastComparison.svg, replace

sjlog using logs/Forecast1, replace
* QL criteria
generate QLvar_GARCH = sum((return2/var_GARCH_Forecast-ln(return2/var_GARCH_Forecast)-1)) if date>=mdy(12,01,2013)
generate QLvar_TGARCH = sum((return2/var_TGARCH_Forecast-ln(return2/var_TGARCH_Forecast)-1)) if date>=mdy(12,01,2013)
generate QLvar_GJR = sum((return2/var_GJR_Forecast-ln(return2/var_GJR_Forecast)-1)) if date>=mdy(12,01,2013)
summarize QLvar_GARCH QLvar_TGARCH QLvar_GJR
tsset newdate
sjlog close, replace

************ ADDITIONAL GARCH MODELS **********************
* PARCH(1,1)
sjlog using logs/PGARCH, replace
arch return, parch(1) pgarch(1) nolog
sjlog close, replace

* NGARCH(2,1)
*arch return, narch(1/2) garch(2) nolog -- works
sjlog using logs/NGARCH, replace
arch return, narch(1/2) garch(1) arch0(xbwt) nolog
sjlog close, replace

* NGARCHK(2,1)
sjlog using logs/NGARCHK, replace
arch return, narchk(1/2) garch(1) nolog
sjlog close, replace

