clear all
set more off
set scheme s2manual

use index

* parametric VaR on SP500 normal distribution
sjlog using logs/VaRN, replace
quietly summarize SP500
local n = r(N)
scalar mean = r(mean)
gmm (SP500 - {mu})((SP500 - {mu})^2 - (`n'-1)/(`n')*{v}), onestep winitial(identity) vce(robust)
sjlog close, replace

sjlog using logs/VaRN1, replace
nlcom (VaR95: _b[mu:_cons] + sqrt(_b[v:_cons])*invnormal(0.05))  ///
	(VaR97: _b[mu:_cons] + sqrt(_b[v:_cons])*invnormal(0.03))  ///
	(VaR99: _b[mu:_cons] + sqrt(_b[v:_cons])*invnormal(0.01)), noheader
sjlog close, replace

sjlog using logs/VaRN3, replace
* weekly VaR
nlcom (VaR95: (_b[mu:_cons] + sqrt(_b[v:_cons])*invnormal(0.05))*sqrt(5))    ///
	(VaR97: (_b[mu:_cons] + sqrt(_b[v:_cons])*invnormal(0.03))*sqrt(5))  ///
	(VaR99: (_b[mu:_cons] + sqrt(_b[v:_cons])*invnormal(0.01))*sqrt(5)), noheader
* monthly VaR
nlcom (VaR95: (_b[mu:_cons] + sqrt(_b[v:_cons])*invnormal(0.05))*sqrt(22))  ///
	(VaR97: (_b[mu:_cons] + sqrt(_b[v:_cons])*invnormal(0.03))*sqrt(22))  ///
	(VaR99: (_b[mu:_cons] + sqrt(_b[v:_cons])*invnormal(0.01))*sqrt(22)), noheader
sjlog close, replace

sjlog using logs/VaRT, replace
* parametric VaR on SP 500 Student t distribution
nlcom (VaR95: _b[mu:_cons] + sqrt(_b[v:_cons])*invt(5,0.05))  ///
	(VaR97: _b[mu:_cons] + sqrt(_b[v:_cons])*invt(5,0.03))  ///
	(VaR99: _b[mu:_cons] + sqrt(_b[v:_cons])*invt(5,0.01)), noheader
sjlog close, replace

* VaR on GARCH(1,1)
sjlog using logs/VaRGARCH, replace
arch SP500, arch(1) garch(1)
predict variance, variance
generate VaRGARCH95 = mean+variance^0.5*invnormal(0.05)
generate VaRGARCH97 = mean+variance^0.5*invnormal(0.03)
generate VaRGARCH99 = mean+variance^0.5*invnormal(0.01)
sjlog close, replace

sjlog using logs/VaRGARCH_graph1, replace
tsset date
tsline SP500 VaRGARCH95 VaRGARCH97 VaRGARCH99
sjlog close, replace
tsline SP500 VaRGARCH95 VaRGARCH97 VaRGARCH99, lcolor(black gs5 gs10 gs15)
graph export eps/VaRGARCH.eps, replace
graph export svg/VaRGARCH.svg, replace

* compute VaR standard errors
sjlog using logs/VaRGARCHSE, replace
tsset newdate
predict residuals, residuals
predictnl VaR = mean+([ARCH]_b[_cons]+ [ARCH]_b[L.arch]*L.residuals^2+[ARCH]_b[L.garch]*L.variance)^0.5*invnormal(0.05), se(VaR_SE)
generate VaRPlusSE = VaR+VaR_SE
generate VaRMinusSE = VaR-VaR_SE
sjlog close, replace

sjlog using logs/VaRGARCHSEChart, replace
tsset date
tsline SP500 VaR VaRPlusSE VaRMinusSE
sjlog close, replace
tsline SP500 VaR VaRPlusSE VaRMinusSE, lcolor(black gs5 gs10 gs15)
graph export eps/VaRGARCHSE.eps, replace
graph export svg/VaRGARCHSE.svg, replace

* VaR on GARCH(1,1) and Student-t distribution
sjlog using logs/VaRGARCHT, replace
tsset newdate
drop variance
arch SP500, arch(1) garch(1) distribution(t) nolog
predict variance, variance
generate VaRGARCHT95 = mean+variance^0.5*invt(5,0.05)
generate VaRGARCHT97 = mean+variance^0.5*invt(5,0.03)
generate VaRGARCHT99 = mean+variance^0.5*invt(5,0.01)
sjlog close, replace

sjlog using logs/VaRGARCHTChart, replace
tsset date
tsline SP500 VaRGARCH95 VaRGARCHT95, lcolor(black gs10 gs5)
sjlog close, replace
graph export eps/VaRGARCHT.eps, replace
graph export svg/VaRGARCHT.svg, replace

sjlog using logs/VaRP, replace
quietly summarize SP500
local n = r(N)
matrix A = (r(mean), r(Var))
quietly summarize FTSE100
matrix B = (r(mean), r(Var))
quietly correlate SP500 FTSE100
matrix init = A, B, r(rho)
gmm (SP500-{mu_SP500})   ///
        ((SP500-{mu_SP500})^2 - (`n')/(`n'-1)*{v_SP500})           ///
        (FTSE100-{mu_FTSE100})  ///
        ((FTSE100-{mu_FTSE100})^2 - (`n')/(`n'-1)*{v_FTSE100}) ///
        ((SP500-{mu_SP500})*(FTSE100-{mu_FTSE100})/(sqrt({v_SP500})*sqrt({v_FTSE100}))-{rho}), ///
        onestep winitial(identity) vce(robust) from(init)
sjlog close, replace

sjlog using logs/VaRP1, replace
nlcom (VaRP: -sqrt((_b[mu_SP500:_cons] + sqrt(_b[v_SP500:_cons]))*   ///
	(_b[mu_FTSE100:_cons] + sqrt(_b[v_FTSE100:_cons]))* ///
	invnormal(0.95)*_b[rho:_cons]))
nlcom (VaRP: -sqrt((_b[mu_SP500:_cons] + sqrt(_b[v_SP500:_cons]))*        ///
	(_b[mu_FTSE100:_cons] + sqrt(_b[v_FTSE100:_cons]))*       ///
	invnormal(0.97)*_b[rho:_cons]))
nlcom (VaRP: -sqrt((_b[mu_SP500:_cons] + sqrt(_b[v_SP500:_cons]))*     ///
	(_b[mu_FTSE100:_cons] + sqrt(_b[v_FTSE100:_cons]))*   ///
	invnormal(0.99)*_b[rho:_cons]))
sjlog close, replace

* portfolio VaR with DCC
sjlog using logs/VaRPDCC, replace
tsset newdate
mgarch dcc SP500 FTSE100, arch(1) garch(1) nolog
sjlog close, replace

sjlog using logs/VaRPDCC1, replace
predict variance*, variance
quietly summarize SP500
generate mean1 = r(mean)
quietly summarize FTSE100
generate mean2 = r(mean)
generate meanP = mean1*0.5+mean2*0.5
generate VaRPDCC95 = meanP+sqrt(variance_FTSE100_SP500)*invnormal(0.05)
generate VaRPDCC97 = meanP+sqrt(variance_FTSE100_SP500)*invnormal(0.03)
generate VaRPDCC99 = meanP+sqrt(variance_FTSE100_SP500)*invnormal(0.01)
sjlog close, replace

sjlog using logs/VaRPDCC2, replace
generate returnP = SP500*0.5+FTSE100*0.5
tsset date
tsline returnP VaRPDCC95 VaRPDCC97 VaRPDCC99
sjlog close, replace
tsline returnP VaRPDCC95 VaRPDCC97 VaRPDCC99, lcolor(black gs15 gs10 gs5)
graph export eps/VaRGARCHDCC.eps, replace
graph export svg/VaRGARCHDCC.svg, replace

* portfolio VaR with DCC under Student-t distribution
sjlog using logs/VaRPDCCT, replace
tsset newdate
mgarch dcc SP500 FTSE100, arch(1) garch(1) distribution(t) nolog
predict varianceT*, variance
generate VaRPDCCT95 = meanP+sqrt(varianceT_FTSE100_SP500)*invnormal(0.05)
generate VaRPDCCT97 = meanP+sqrt(varianceT_FTSE100_SP500)*invnormal(0.03)
generate VaRPDCCT99 = meanP+sqrt(varianceT_FTSE100_SP500)*invnormal(0.01)
tsset date
tsline returnP VaRPDCCT95 VaRPDCCT97 VaRPDCCT99
sjlog close, replace
tsline returnP VaRPDCCT95 VaRPDCCT97 VaRPDCCT99, lcolor(black gs15 gs10 gs5)
graph export eps/VaRGARCHDCCT.eps, replace
graph export svg/VaRGARCHDCCT.svg, replace

* historical simulation VaR
sjlog using logs/HistoricalRet, replace
histogram SP500
sjlog close, replace
graph export eps/HistoricalRet.eps, replace
graph export svg/HistoricalRet.svg, replace

sjlog using logs/VaRHist, replace
centile SP500, centile(1,3,5)
sjlog close, replace

sjlog using logs/VaRHist2, replace
centile SP500 if date >= mdy(01,01,2000), centile(1,3,5)
sjlog close, replace

sjlog using logs/MCP, replace
generate returnportfolio = SP500*0.5+FTSE100*0.5
histogram returnportfolio, normal width(0.001)
sjlog close, replace
graph export eps/HistoricalRetPortfolio.eps, replace
graph export svg/HistoricalRetPortfolio.svg, replace

sjlog using logs/MCPVaR, replace
_pctile returnportfolio, p(1,3,5)
return list
sjlog close, replace

* MONTE CARLO SIMULATION
use index, clear
sjlog using logs/MCN, replace
set obs 10000
quietly summarize SP500
scalar mean = r(mean)
scalar std = r(sd)
set seed 1
generate simulatedreturns = rnormal(mean, std)
sjlog close, replace

sjlog using logs/SimulatedRetN, replace
histogram simulatedreturns
sjlog close, replace
graph export eps/SimulatedRetN.eps, replace
graph export svg/SimulatedRetN.svg, replace

sjlog using logs/VaRMCN, replace
centile simulatedreturns, centile(1,3,5)
sjlog close, replace

* Monte Carlo simulation based on Student-t with 5 degrees of freedom
sjlog using logs/MCT, replace
set seed 1
generate simulatedT = rt(5)
generate simulatedT_LocationScale = simulatedT*std+mean
histogram simulatedT_LocationScale
sjlog close, replace
graph export eps/SimulatedRetT.eps, replace
graph export svg/SimulatedRetT.svg, replace

sjlog using logs/VaRMCT, replace
centile simulatedT_LocationScale, centile(1,3,5)
sjlog close, replace

* Monte Carlo bivariate
sjlog using logs/MCBiv, replace
summarize SP500
matrix mean1 = r(mean)
matrix sd1 = r(sd)
summarize FTSE100
matrix mean2 = r(mean)
matrix sd2 = r(sd)
matrix meanV = (mean1,mean2)
matrix sdV = (sd1,sd2)
correlate SP500 FTSE100
matrix correlation = r(C)
sjlog close, replace

sjlog using logs/MCBiv3, replace
set seed 1
drawnorm SPSimulated FTSESimulated, n(10000) means(meanV) sds(sdV) corr(correlation)
sjlog close, replace

sjlog using logs/MCBiv2, replace
summarize SPSimulated FTSESimulated
correlate SPSimulated FTSESimulated
sjlog close, replace

sjlog using logs/MCBivVaR, replace
generate simulateP = SPSimulated*0.5+FTSESimulated*0.5
histogram simulateP, normal width(0.001)
sjlog close, replace
graph export eps/MCSimulatedP.eps, replace
graph export svg/MCSimulatedP.svg, replace

sjlog using logs/MCBivVaR1, replace
_pctile(simulateP), p(1 3 5)
return list
sjlog close, replace

* expected shortfall
sjlog using logs/ESNormal, replace
scalar ES95 = normalden(invnormal(1-0.95))/(1-0.95)
scalar ES97 = normalden(invnormal(1-0.97))/(1-0.97)
scalar ES99 = normalden(invnormal(1-0.99))/(1-0.99)
display ES95,ES97,ES99
sjlog close, replace

* UNCONDITIONAL TEST
use index, clear
sjlog using logs/UC95, replace
quietly summarize SP500
scalar mean = r(mean)
quietly arch SP500, arch(1) garch(1)
predict variance, variance
scalar confidencelevel = 1 - 0.95
generate VaR = mean+variance^0.5*invnormal(confidencelevel)
generate violations = SP500<VaR
sjlog close, replace

sjlog using logs/UC951, replace
summarize violations
scalar T1 = r(sum)
scalar T = r(N)
scalar empiricalratio = T1/T
display empiricalratio
scalar loglEmpirical = (T-T1)*log(1-empiricalratio)+T1*log(empiricalratio)
scalar loglTheoretical = (T-T1)*log(1-confidencelevel)+T1*log(confidencelevel)
scalar LRUC = 2*(loglTheoretical-loglEmpirical)
display 1-chi2(1,abs(LRUC))
drop VaR violations 
sjlog close, replace

* unconditional coverage at 99%
sjlog using logs/UC99, replace
scalar confidencelevel = 1 - 0.99
generate VaR = mean+variance^0.5*invnormal(confidencelevel)
generate violations = SP500<VaR
summarize violations
scalar T1 = r(sum)
scalar T = r(N)
scalar empiricalratio = T1/T
display empiricalratio
scalar loglEmpirical = (T-T1)*log(1-empiricalratio)+T1*log(empiricalratio)
scalar loglTheoretical = (T-T1)*log(1-confidencelevel)+T1*log(confidencelevel)
scalar LRUC = 2*(loglTheoretical-loglEmpirical)
display 1-chi2(1,abs(LRUC))
drop variance VaR violations
sjlog close, replace

* unconditional coverage Student-t
sjlog using logs/UCT99, replace
quietly arch SP500, arch(1) garch(1) distribution(t)
predict variance, variance
generate VaR = mean+variance^0.5*invt(6,confidencelevel)
generate violations = SP500<VaR
sjlog close, replace

sjlog using logs/UCT992, replace
summarize violations
scalar T1 = r(sum)
scalar T = r(N)
scalar empiricalratio = T1/T
display empiricalratio
scalar loglEmpirical = (T-T1)*log(1-empiricalratio)+T1*log(empiricalratio)
scalar loglTheoretical = (T-T1)*log(1-confidencelevel)+T1*log(confidencelevel)
scalar LRUC = 2*(loglTheoretical-loglEmpirical)
display 1-chi2(1,abs(LRUC))
drop variance VaR violations 
sjlog close, replace

* independence test
sjlog using logs/Ind951, replace
quietly arch SP500, arch(1) garch(1)
predict variance, variance
scalar confidencelevel = 1 - 0.95
generate VaR = mean+variance^0.5*invnormal(confidencelevel)
generate violations = SP500<VaR
generate violations01 = (L.violations==0 & violations==1)
generate violations11 = (L.violations==1 & violations==1)
summarize violations01
scalar T01 = r(sum)
scalar T = r(N)
summarize violations11
scalar T11 = r(sum)
scalar T1 = T01+T11
scalar T0 = T-T1
scalar pi01 = T01/T0
scalar pi11 = T11/T1
scalar pi1 = T1/T
sjlog close, replace

sjlog using logs/Ind952, replace
scalar loglEmpirical = (T0-T01)*log(1-pi01)+T01*log(pi01)+(T1-T11)*log(1-pi11)+T11*log(pi11)
scalar loglTheoretical = (T-T1)*log(1-pi1)+T1*log(pi1)
scalar LRInd = 2*(loglTheoretical-loglEmpirical)
display 1-chi2(1,abs(LRInd))
drop violations01 violations11
sjlog close, replace

* conditional coverage test
sjlog using logs/CC951, replace
generate violations01 = (L.violations==0 & violations==1)
generate violations11 = (L.violations==1 & violations==1)
summarize violations01
scalar T01 = r(sum)
scalar T = r(N)
summarize violations11
scalar T11 = r(sum)
scalar T1 = T01+T11
scalar T0 = T-T1
scalar pi01 = T01/T0
scalar pi11 = T11/T1
scalar pi1 = T1/T
sjlog close, replace

sjlog using logs/CC952, replace
scalar loglEmpirical = (T0-T01)*log(1-pi01)+T01*log(pi01)+(T1-T11)*log(1-pi11)+T11*log(pi11)
scalar loglTheoretical = (T-T1)*log(1-confidencelevel)+T1*log(confidencelevel)
scalar LRCC = 2*(loglTheoretical-loglEmpirical)
display 1-chi2(2,abs(LRCC))
sjlog close, replace

